﻿using FurnitureAssembly;
using FurnitureAssemblyContracts.BindingModels;
using FurnitureAssemblyContracts.BusinessLogicsContracts;
using FurnitureAssemblyContracts.SearchModels;
using FurnitureAssemblyDataModels.Models;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace FurnitureAssemblyView
{
    public partial class FormFurniture : Form
    {
        private readonly ILogger _logger;
        private readonly IFurnitureLogic _logic;
        private int? _id;
        private Dictionary<int, (IWorkpieceModel, int)> _productWorkpieces;
        public int Id { set { _id = value; } }
        public FormFurniture(ILogger<FormFurniture> logger, IFurnitureLogic logic)
         {
         InitializeComponent();
                _logger = logger;
         _logic = logic;
         _productWorkpieces = new Dictionary<int, (IWorkpieceModel, int)>();
         }
    private void FormFurniture_Load(object sender, EventArgs e)
    {
        if (_id.HasValue)
        {
            _logger.LogInformation("Загрузка заготовки");
            try
            {
                var view = _logic.ReadElement(new FurnitureSearchModel { Id =_id.Value });
                if (view != null)
                {
                    textBoxName.Text = view.FurnitureName;
                    textBoxPrice.Text = view.Price.ToString();
                    _productWorkpieces = view.FurnitureWorkpieces ?? new Dictionary<int, (IWorkpieceModel, int)>();
                    LoadData();
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки заготовки");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,
                MessageBoxIcon.Error);
            }
        }
    }
    private void LoadData()
    {
        _logger.LogInformation("Загрузка компонент заготовки");
        try
        {
            if (_productWorkpieces != null)
            {
                dataGridView.Rows.Clear();
                foreach (var pc in _productWorkpieces)
                {
                    dataGridView.Rows.Add(new object[] { pc.Key, pc.Value.Item1.WorkpieceName, pc.Value.Item2 });
                }
                textBoxPrice.Text = CalcPrice().ToString();
            }
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка загрузки компонент заготовки");
            MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,
           MessageBoxIcon.Error);
        }
    }
    private void ButtonAdd_Click(object sender, EventArgs e)
    {
        var service =
       Program.ServiceProvider?.GetService(typeof(FormFurnitureWorkpiece));
        if (service is FormFurnitureWorkpiece form)
        {
        if (form.ShowDialog() == DialogResult.OK)
            {
                if (form.WorkpieceModel == null)
                {
                    return;
                }
                _logger.LogInformation("Добавление нового компонента:{ WorkpieceName} - { Count} ", form.WorkpieceModel.WorkpieceName, form.Count);
                 if (_productWorkpieces.ContainsKey(form.Id))
                {
                    _productWorkpieces[form.Id] = (form.WorkpieceModel,form.Count);
                }
                else
                {
                    _productWorkpieces.Add(form.Id, (form.WorkpieceModel,form.Count));
                }
                LoadData();
            }
        }
    }
    private void ButtonUpd_Click(object sender, EventArgs e)
    {
        if (dataGridView.SelectedRows.Count == 1)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormFurnitureWorkpiece));
            if (service is FormFurnitureWorkpiece form)
            {
                int id = Convert.ToInt32(dataGridView.SelectedRows[0].Cells[0].Value);
                form.Id = id;
                form.Count = _productWorkpieces[id].Item2;
                if (form.ShowDialog() == DialogResult.OK)
                {
                    if (form.WorkpieceModel == null)
                    {
                        return;
                    }
                    _logger.LogInformation("Изменение компонента:{ WorkpieceName} - { Count} ", form.WorkpieceModel.WorkpieceName, form.Count);
                 _productWorkpieces[form.Id] = (form.WorkpieceModel,form.Count);
                    LoadData();
                }
            }
        }
    }
    private void ButtonDel_Click(object sender, EventArgs e)
    {
        if (dataGridView.SelectedRows.Count == 1)
        {
            if (MessageBox.Show("Удалить запись?", "Вопрос",
           MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                try
                {
                    _logger.LogInformation("Удаление компонента: { WorkpieceName} - { Count} ", dataGridView.SelectedRows[0].Cells[1].Value);
               
            _productWorkpieces?.Remove(Convert.ToInt32(dataGridView.SelectedRows[0].Cells[0].Value));
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "Ошибка",
                   MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                LoadData();
            }
        }
    }
    private void ButtonRef_Click(object sender, EventArgs e)
    {
        LoadData();
    }
    private void ButtonSave_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(textBoxName.Text))
        {
            MessageBox.Show("Заполните название", "Ошибка",
           MessageBoxButtons.OK, MessageBoxIcon.Error);
            return;
        }
        if (string.IsNullOrEmpty(textBoxPrice.Text))
        {
            MessageBox.Show("Заполните цену", "Ошибка", MessageBoxButtons.OK,
           MessageBoxIcon.Error);
            return;
        }
        if (_productWorkpieces == null || _productWorkpieces.Count == 0)
        {
            MessageBox.Show("Заполните компоненты", "Ошибка",
           MessageBoxButtons.OK, MessageBoxIcon.Error);
            return;
        }
        _logger.LogInformation("Сохранение заготовки");
        try
        {
            var model = new FurnitureBindingModel
            {
                Id = _id ?? 0, FurnitureName = textBoxName.Text,
                Price = Convert.ToDouble(textBoxPrice.Text),
                FurnitureWorkpieces = _productWorkpieces
            };
            var operationResult = _id.HasValue ? _logic.Update(model) :
           _logic.Create(model);
            if (!operationResult)
            {
                throw new Exception("Ошибка при сохранении. Дополнительная  информация в логах.");
            }
            MessageBox.Show("Сохранение прошло успешно", "Сообщение",
           MessageBoxButtons.OK, MessageBoxIcon.Information);
            DialogResult = DialogResult.OK;
            Close();
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка сохранения заготовки");
        MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,MessageBoxIcon.Error);
        }
    }
    private void ButtonCancel_Click(object sender, EventArgs e)
    {
        DialogResult = DialogResult.Cancel;
        Close();
    }
    private double CalcPrice()
    {
        double price = 0;
        foreach (var elem in _productWorkpieces)
        {
            price += ((elem.Value.Item1?.Cost ?? 0) * elem.Value.Item2);
        }
        return Math.Round(price * 1.1, 2);
    }
    }
}
